# 交易对信息错误
EXCHANGE_INFO_ERROR = ['EXCHANGE_INFO_ERROR', 'Transaction rules and transaction pair errors']
# 历史K线时间起点错误
CANDLE_START_ERROR = ['CANDLE_START_ERROR', 'History candle start ts error']
# 历史K线时间终点错误
CANDLE_END_ERROR = ['CANDLE_END_ERROR', 'History candle end ts error']
# 历史K线长度错误
CANDLE_LENGTH_ERROR = ['CANDLE_LENGTH_ERROR', 'History candle length error']
# 历史K线时间间隔错误
CANDLE_INTERVAL_ERROR = ['CANDLE_INTERVAL_ERROR', 'History candle ts interval error']
